/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

/* Parts of this file were written by Frans Kaashoek and Kevin Fu. */

%#include "bigint.h"
%#include "sfs_prot.h"
%#include "vervec.h"

%inline bool
%rpc_traverse (rpc_clear_t &, bigint &obj)
%{
%  obj = 0;
%  return true;
%}

const CFS_FHSIZE = 20;
const CFS_NDIR = 128;

enum ftypero {
  CFSREG      = 1,
  CFSREG_EXEC = 2,  /* Regular, executable file */
  CFSDIR      = 3, 
  CFSDIR_OPAQ = 4,
  CFSLNK      = 5
};

struct rwattr {
  uint64 size;
  uint32 mode;
  uint32 uid;
  uint32 gid;
  specdata3 rdev;
};

struct cfs_inode_lnk {
  nfspath3 path;
  uint32 nlink;
  nfstime3 mtime;
  nfstime3 ctime;
  opaque mver<>;
  nfspath3 dest;
  rwattr attr;
};

struct cfs_inode_reg {
  nfspath3 path;
  uint32 nlink;
  nfstime3 mtime;
  nfstime3 ctime;
  rwattr attr;
  uint64 mstart;   /* mark range of data modified */
  uint64 mend;
  bigint parent; /* directory's .. reference */
  bigint direct<CFS_NDIR>;
  bigint indirect;
  bigint double_indirect;
  bigint triple_indirect;
  opaque mver<>;
};

union cfs_inode switch (ftypero type) {
case CFSLNK:
  cfs_inode_lnk lnk;
default:
  cfs_inode_reg reg;
};

struct cfs_indirect {
  bigint handles<>;
};

struct cfs_dirent {
  bigint fh;
  string name<>;
  cfs_dirent *nextentry;
};

struct cfs_directory {
  uint32 count;
  cfs_dirent *entries;
  bool eof;
};

struct cfs_rpc_dirent {
  bigint fh;
  string name<>;
  uint64 cookie;
  cfs_rpc_dirent *nextentry;
};

struct cfs_dirlist {
  cfs_rpc_dirent *entries;
  bool eof;
};

enum dtype {
   CFS_INODE      = 0,
   CFS_FILEBLK    = 1, /* File data */
   CFS_DIRBLK     = 2, /* Directory data */
   CFS_INDIR      = 3  /* Indirect data pointer block */
};

union cfs_data switch (dtype type) {
 case CFS_INODE:
   cfs_inode inode;
 case CFS_FILEBLK:
   opaque data<>;
 case CFS_DIRBLK:
   cfs_directory dir;
 case CFS_INDIR:
   cfs_indirect indir;
 default:
   void;
};


