/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#include "cfsrw_prot.h"

void
cfscall::memop (bool free)
{
  switch (_proc) {
    case CFSRWPROC_NULL:
      _arg = 0;
      _res = 0;
      break;
    case CFSRWPROC_GETATTR:
      if (free) {
	delete static_cast<cfsrw_getattr_arg *> (_arg);
	delete static_cast<cfsrw_getattr_res *> (_res);
      }
      else {
	_arg = New cfsrw_getattr_arg;
        _res = New cfsrw_getattr_res;
      }
      break;
    case CFSRWPROC_GETPARENT:
      if (free) {
	delete static_cast<cfsrw_getparent_arg *> (_arg);
	delete static_cast<cfsrw_getparent_res *> (_res);
      }
      else {
	_arg = New cfsrw_getparent_arg;
        _res = New cfsrw_getparent_res;
      }
      break;
    case CFSRWPROC_LOOKUP:
      if (free) {
	delete static_cast<cfsrw_lookup_arg *> (_arg);
	delete static_cast<cfsrw_lookup_res *> (_res);
      }
      else {
	_arg = New cfsrw_lookup_arg;
        _res = New cfsrw_lookup_res;
      }
      break;
    case CFSRWPROC_READ:
      if (free) {
	delete static_cast<cfsrw_read_arg *> (_arg);
	delete static_cast<cfsrw_read_res *> (_res);
      }
      else {
	_arg = New cfsrw_read_arg;
        _res = New cfsrw_read_res;
      }
      break;
    case CFSRWPROC_READLINK:
      if (free) {
	delete static_cast<cfsrw_readlink_arg *> (_arg);
	delete static_cast<cfsrw_readlink_res *> (_res);
      }
      else {
	_arg = New cfsrw_readlink_arg;
        _res = New cfsrw_readlink_res;
      }
      break;
    case CFSRWPROC_READDIR:
      if (free) {
	delete static_cast<cfsrw_readdir_arg *> (_arg);
	delete static_cast<cfsrw_readdir_res *> (_res);
      }
      else {
	_arg = New cfsrw_readdir_arg;
        _res = New cfsrw_readdir_res;
      }
      break;
    case CFSRWPROC_SETATTR:
      if (free) {
	delete static_cast<cfsrw_setattr_arg *> (_arg);
	delete static_cast<cfsrw_nores_res *> (_res);
      }
      else {
	_arg = New cfsrw_setattr_arg;
        _res = New cfsrw_nores_res;
      }
      break;
    case CFSRWPROC_UNLINK:
      if (free) {
	delete static_cast<cfsrw_unlink_arg *> (_arg);
	delete static_cast<cfsrw_nores_res *> (_res);
      }
      else {
	_arg = New cfsrw_unlink_arg;
        _res = New cfsrw_nores_res;
      }
      break;
    case CFSRWPROC_WRITE:
      if (free) {
	delete static_cast<cfsrw_write_arg *> (_arg);
	delete static_cast<cfsrw_nores_res *> (_res);
      }
      else {
	_arg = New cfsrw_write_arg;
        _res = New cfsrw_nores_res;
      }
      break;
    case CFSRWPROC_CREATE:
    case CFSRWPROC_MKDIR:
      if (free) {
	delete static_cast<cfsrw_create_arg *> (_arg);
	delete static_cast<cfsrw_create_res *> (_res);
      }
      else {
	_arg = New cfsrw_create_arg;
        _res = New cfsrw_create_res;
      }
      break;
    case CFSRWPROC_SYMLINK:
      if (free) {
	delete static_cast<cfsrw_symlink_arg *> (_arg);
	delete static_cast<cfsrw_symlink_res *> (_res);
      }
      else {
	_arg = New cfsrw_symlink_arg;
        _res = New cfsrw_symlink_res;
      }
      break;
    case CFSRWPROC_LINK:
      if (free) {
	delete static_cast<cfsrw_link_arg *> (_arg);
	delete static_cast<cfsrw_nores_res *> (_res);
      }
      else {
	_arg = New cfsrw_link_arg;
        _res = New cfsrw_nores_res;
      }
      break;
    case CFSRWPROC_RENAME:
      if (free) {
	delete static_cast<cfsrw_rename_arg *> (_arg);
	delete static_cast<cfsrw_nores_res *> (_res);
      }
      else {
	_arg = New cfsrw_rename_arg;
        _res = New cfsrw_nores_res;
      }
      break;
    case CFSRWPROC_EXCL:
      if (free) {
	delete static_cast<cfsrw_excl_arg *> (_arg);
	delete static_cast<cfsrw_nores_res *> (_res);
      }
      else {
	_arg = New cfsrw_excl_arg;
        _res = New cfsrw_nores_res;
      }
      break;
    default:
      fatal ("unknown cfsrw operation\n");
      assert (0);
      break;
  }
}

