
/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA
 */

#ifndef CFSRW_SNAPSHOT_SPEC_H
#define CFSRW_SNAPSHOT_SPEC_H

#include "cfsrw.h"

class snapshot_obj {
public:
  virtual bigint location () const = 0;
  virtual bigint prev ()     const = 0;
  virtual vervec version () const  = 0;
};

typedef callback<void, bigint, ptr<snapshot_obj> >::ptr cb_snapshot;

class snapshot_operation : public virtual refcount {
public:
  static int init ();

  static void load_snapshot (bigint location,
                             callback<void, bigint, ptr<snapshot_obj> >::ref);
  static void build_snapshot (ptr<snapshot_obj> prev, bigint loc, vervec start,
                              callback<void, bigint, ptr<snapshot_obj> >::ref);
  static void build_snapshot (bigint root, vervec start,
                              callback<void, bigint, ptr<snapshot_obj> >::ref);

  static void find_parent (ptr<snapshot_obj>, bigint object,
                           cfsrw_states::cb_fh);
  static void lookup (ptr<snapshot_obj>, bigint dir, str name,
                      cfsrw_states::cb_lookup);
  static void find_inode (ptr<snapshot_obj>, bigint object,
                          ptr<cfsrw_states::attr>, cfsrw_states::cb_attr);
  static void readdir (ptr<snapshot_obj>, bigint dir,
		       ptr<cfsrw_states::readdir>, cfsrw_states::cb_readdir);
  static void read (ptr<snapshot_obj>,
                    ptr<cfsrw_states::read>, cfsrw_states::cb_read cb);
};

#endif

