/*
 *  Copyright (C) 1998  David Mazieres (dm@uun.org)
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *  DEALINGS IN THE SOFTWARE.
 */

#include "arpc.h"
#include "sfsclient.h"
#include "sfsmisc.h"
#include "vervec.h"

class cfsrw_attr_cache {
  struct access_dat {
    u_int32_t mask;
    u_int32_t perm;
    access_dat (u_int32_t m, u_int32_t p) : mask (m), perm (p) {}
  };

public:
  struct attr_dat {
    cfsrw_attr_cache *const cache;
    const nfs_fh3 fh;
    fattr3 attr;
    qhash<sfs_aid, access_dat> access;
    ihash_entry<attr_dat> fhlink;
    tailq_entry<attr_dat> lrulink;
    attr_dat (cfsrw_attr_cache *c, const nfs_fh3 &f, const fattr3 *a);
    ~attr_dat ();
    void touch ();
    void set (const fattr3 *a, const wcc_attr *w);
  };

private:
  friend class attr_dat;
  ihash<const nfs_fh3, attr_dat, &attr_dat::fh, &attr_dat::fhlink> attrs;
  static void remove_aid (sfs_aid aid, attr_dat *ad)
    { ad->access.remove (aid); }
  vervec version;
  timespec ctime;

public:
  ~cfsrw_attr_cache () { attrs.deleteall (); }
  void flush_attr () { attrs.deleteall (); }
  void flush_access (sfs_aid aid) { attrs.traverse (wrap (remove_aid, aid)); }
  void flush_access (const nfs_fh3 &fh, sfs_aid aid);
  
  void current_version (const vervec &v) { version = v; ctime = tsnow; }
  timespec cache_time () const { return ctime; }
  vervec current_version () const { return version; }

  void attr_enter (const nfs_fh3 &, const fattr3 *, const wcc_attr *);
  const fattr3 *attr_lookup (const vervec &, const nfs_fh3 &);
  const fattr3 *last_known_attr_lookup (const nfs_fh3 &);

  void access_enter (const nfs_fh3 &, sfs_aid aid,
                     u_int32_t mask, u_int32_t perm);
  int32_t access_lookup (const vervec &, const nfs_fh3 &,
                         sfs_aid aid, u_int32_t mask);
};

