/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA
 */

#ifndef FCACHE_H
#define FCACHE_H

#include "arpc.h"
#include "aiod.h"
#include "nfstrans.h"
#include "sfsclient.h"
#include "qhash.h"
#include "itree.h"
#include "crypt.h"
#include "list.h"
#include "lrucache.h"
#include "ranges.h"
#include "vervec.h"

class file_cache {
  friend class read_obj;
public:
  nfs_fh3 fh;
  int status;
  vervec version;
  fattr3 fa;
  ptr<aiofh> afh;
  int outstanding_ops;
  uint64 mstart;
  uint64 mend;
  vec<nfscall *> rpcs;

private:
  static const int fcache_open  = 0;
  static const int fcache_fetch = 1;
  static const int fcache_idle  = 2;
  static const int fcache_dirty = 3;
  static const int fcache_flush = 4;
  static const int fcache_error = 5;

  vec<uint64> pri;
  ranges *rcv;
  ranges *req;
  void cr () {
    if (rcv) {
      delete rcv;
      delete req;
    }
    rcv = 0;
    req = 0;
  }
  void ar () {
    assert (!rcv && !req);
    rcv = New ranges ();
    req = New ranges ();
  }

public:
  file_cache (nfs_fh3 fh)
    : fh(fh), status(fcache_open), afh(0), outstanding_ops(0),
      mstart(0), mend(0), rcv(0), req(0)
  {
    fa.mtime.seconds = 0;
    fa.mtime.nseconds = 0;
  }
  ~file_cache () { if (rcv) delete rcv; assert (rpcs.size() == 0); }

  bool is_idle ()    const { return status == fcache_idle; }
  bool is_open ()    const { return status == fcache_open; }
  bool is_fetch ()   const { return status == fcache_fetch; }
  bool is_flush ()   const { return status == fcache_flush; }
  bool is_dirty ()   const { return status == fcache_dirty; }
  bool is_error ()   const { return status == fcache_error; }

  void idle ()    { cr(); status = fcache_idle; }
  void open ()    { cr(); status = fcache_open; }
  void fetch ()   { ar(); status = fcache_fetch; }
  void flush ()	  { cr(); status = fcache_flush; }
  void dirty ()   { cr(); status = fcache_dirty; }
  void error ()   { cr(); status = fcache_error; }

  void outstanding_op () { outstanding_ops++; }
  void outstanding_op_done () { outstanding_ops--; }
  bool being_modified () const { return outstanding_ops > 0; }

  bool received (uint64 off, uint64 size) const {
    if (is_fetch () && (!rcv || !rcv->filled (off, size)))
      return false;
    return true;
  }
  void want (uint64 off, uint64 size, unsigned rtpref) {
    for (uint64 i=off; i<off+size; i+=rtpref)
      pri.push_back (i);
  }

};

#endif

