/*
 *  Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA
 */

#ifndef CFSRW_NFSTRANS_H
#define CFSRW_NFSTRANS_H

#include "nfs3_prot.h"

inline bool
nfs_constop (u_int32_t proc)
{
  switch (proc) {
  case NFSPROC3_SETATTR:
  case NFSPROC3_WRITE:
  case NFSPROC3_CREATE:
  case NFSPROC3_MKDIR:
  case NFSPROC3_SYMLINK:
  case NFSPROC3_MKNOD:
  case NFSPROC3_REMOVE:
  case NFSPROC3_RMDIR:
  case NFSPROC3_RENAME:
  case NFSPROC3_LINK:
    return false;
  default:
  case NFSPROC3_COMMIT: // sic
    return true;
  }
}

struct xfh {
  nfs_fh3 *fh;
};
typedef vec<xfh, 2> xfhvec;

DUMBTRAVERSE (xfhvec)

bool rpc_traverse (xfh &xv, nfs_fh3 &fh);
void nfs3_getxfh (xfhvec *xvp, u_int32_t proc, void *argp, void *resp);

struct xfattr {
  nfs_fh3 *fh;
  fattr3 *fattr;
  wcc_attr *wattr;
  wcc_data *wdata;

  void set_wcc (wcc_data *wd) {
    wdata = wd;
    if (wd->before.present)
      wattr = wd->before.attributes;
    if (wd->after.present)
      fattr = reinterpret_cast<fattr3*>(wd->after.attributes.addr());
  }

  xfattr () : fh (NULL), fattr (NULL), wattr (NULL), wdata (NULL) {}
};
typedef vec<xfattr, 2> xfattrvec;

DUMBTRAVERSE (xfattrvec)

inline bool
rpc_traverse (xfattrvec &xv, fattr3 &obj)
{
  assert (!xv[0].fattr);
  xv[0].fattr = &obj;
  return true;
}

inline bool
rpc_traverse (xfattrvec &xv, wcc_data &obj)
{
  assert (!xv[0].wdata);
  xv[0].set_wcc (&obj);
  return true;
}

bool rpc_traverse (xfattrvec &xv, lookup3resok &obj);
bool rpc_traverse (xfattrvec &xv, diropres3ok &obj);
bool rpc_traverse (xfattrvec &xv, dirlist3 &obj);
bool rpc_traverse (xfattrvec &xv, entryplus3 &obj);
bool rpc_traverse (xfattrvec &xv, rename3wcc &obj);
bool rpc_traverse (xfattrvec &xv, link3wcc &obj);
void nfs3_getxfattr (xfattrvec *xvp, u_int32_t proc, void *argp, void *resp);

#endif

