/*
 * Check that the file mtime does *not* change after a
 * link, rename, or unlink.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

void check_ret(int ok, char *msg);
void check_mtime(char *filename, time_t good, char *msg);
time_t get_mtime(char *filename);
void cleanup(int status);

extern int errno;
char f1[256], f2[256], f3[256];

main()
{
  int fd, cc, ret;
  time_t t0;

  sprintf(f1, "mtime-1-%d", getpid());
  sprintf(f2, "mtime-2-%d", getpid());
  sprintf(f3, "mtime-3-%d", getpid());

  fd = creat(f1, 0666);
  check_ret(fd >= 0, "creat failed");
  close(fd);
  t0 = get_mtime(f1);

  sleep(1);

  check_mtime(f1, t0, "initial");

  check_ret(link(f1, f2) == 0, "link");
  sleep(1);
  check_mtime(f1, t0, "after link");
  check_mtime(f2, t0, "after link");

  check_ret(rename(f1, f3) == 0, "rename");
  sleep(1);
  check_mtime(f2, t0, "after rename");
  check_mtime(f3, t0, "after rename");

  check_ret(unlink(f2) == 0, "unlink");
  sleep(1);
  check_mtime(f3, t0, "after unlink");

  check_ret(unlink(f3) == 0, "unlink");

  cleanup(0);
}

void
check_ret(int ok, char *msg)
{
  if(ok == 0){
    fprintf(stderr, "mtime1 failure: %s: %s\n", msg, strerror(errno));
    cleanup(1);
  }
}

void
check_mtime(char *file, time_t t, char *msg)
{
  int fd;
  struct stat sb;

  /*
   * Actually open the file to avoid client-cached attributes.
   */
  fd = open(file, 0);
  if(fd < 0){
    fprintf(stderr, "mtime1 open %s: %s\n", file, strerror(errno));
    cleanup(1);
  }

  if(fstat(fd, &sb) != 0){
    fprintf(stderr, "mtime1 fstat: %s\n", strerror(errno));
    cleanup(1);
  }

  if(sb.st_mtime != t){
    fprintf(stderr, "mtime1 time check failed: %s\n", msg);
    fprintf(stderr, "mtime1 wanted %u, got %u\n",
            t, sb.st_mtime);
    cleanup(1);
  }

  close(fd);
}

time_t
get_mtime(char *file)
{
  int fd;
  struct stat sb;

  /*
   * Actually open the file to avoid client-cached attributes.
   */
  fd = open(file, 0);
  if(fd < 0){
    fprintf(stderr, "mtime1 open %s: %s\n", file, strerror(errno));
    cleanup(1);
  }

  if(fstat(fd, &sb) != 0){
    fprintf(stderr, "mtime1 fstat: %s\n", strerror(errno));
    cleanup(1);
  }

  close(fd);

  return(sb.st_mtime);
}

void
cleanup(int status)
{
  if(f1[0])
    unlink(f1);
  if(f2[0])
    unlink(f2);
  if(f3[0])
    unlink(f3);
  exit(status);
}
