/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef DEFINC_H
#define DEFINC_H

#include "async.h"
#include "arpc.h"
#include "refcnt.h"
#include "xdrmisc.h"
#include "rpctypes.h"
#include "sfscrypt.h"
#include "bigint.h"
#include "sfsmisc.h"
#include "sha1.h"
#include "str.h"

#define ENV(x,v) getenv(x)?atoi(getenv(x)):v

inline bigint
compute_sha1_hash (const void *buf, size_t buflen)
{
  char h[sha1::hashsize];
  bzero(h, sha1::hashsize);
  sha1_hash (h, buf, buflen);

  bigint n;
  mpz_set_rawmag_be(&n, h, sha1::hashsize);  // For big endian
  return n;
}

enum log_status {
  LOG_OK             = 0,
  LOGERR_ERR         = 1,
  LOGERR_INSERT      = 2,
  LOGERR_NOVIEW      = 100,
  LOGERR_NORECORD    = 101,
  LOGERR_NOLOGHEAD   = 102,
  LOGERR_NOTLOGHEAD  = 103,
  LOGERR_CORRUPTLOG  = 104,
  LOGERR_CONFLICT    = 105,
  LOGERR_OUTOFORDER  = 106,
};

typedef callback<void, log_status>::ptr cb_logstat_t;

#endif

