/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef LOG_H
#define LOG_H

#include "record.h"
#include "ns.h"

typedef callback<void, sfs_pubkey2, sfs_sig2, log_status>::ref cfsrw_sign_cb_t;
typedef callback<void, const str&, cfsrw_sign_cb_t>::ptr cfsrw_sign_t;

class log : public virtual refcount 
{
private:
  bigint _head;
  bool _readonly;
  bigint _root;
  bigint _last_id;
  uint _last_seq;
  bigint _snapshot;
  bigint _new_snapshot;

  bool _refreshed;
  bool _loading;
  vec<cb_logstat_t> _appends_cb;
  vec<recordp> _appends_e;
  ptr<log_record> _head_record;

  bool _locked;
  cb_logstat_t _release_cb;

  int _stores;
  int _failed_stores;
  vec<cb_logstat_t> _store_cbs;

  cfsrw_sign_t sign;

  void init ();

  void store_head_after_append (log_status);
  void head_stored (ptr<vec<cb_logstat_t> >, ptr<head_log_record>, log_status);
  void do_append (log_status);
  
  void store_record (bigint, const str&, cb_logstat_t, recordp e);
  void store_record_cb (cb_logstat_t, ns_stat, bigint);
  void get_record_cb (cb_recordp, int, str);

  void append_log_cb1 (recordp, cb_logstat_t, recordp);
  void append_log_cb2 (bigint, cb_logstat_t, recordp, log_status);

  void load_head (recordp);
  void load_head_cb (cb_logstat_t cb, recordp);
  void store_head (recordp, const str, cb_logstat_t,
                   sfs_pubkey2, sfs_sig2, log_status);
  
public:
  log (bigint, cb_logstat_t cb, cfsrw_sign_t);

  bigint head () const { return _head; }
  bigint root () const { return _root; }
  bigint last_id () const { return _last_id; }
  uint last_seq () const { return _last_seq; }
  bool readonly () const { return _readonly; }
  
  void append (const vervec &v, recordp, cb_logstat_t);

  void snapshot (bigint s);
  bigint snapshot () const { return _snapshot; }

  void lock () { _locked = true; }
  void sync (cb_logstat_t cb);
  void release (cb_logstat_t cb);
  bool locked () const { return _locked; }
};

#endif

