/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef NS_H
#define NS_H

#include "async.h"
#include "sfscrypt.h"
#include "record.h"

struct ns_block {
  char *data;
  size_t len;

  ~ns_block () { delete [] data; }
  ns_block (const char *buf, size_t buflen)
    : data (New char[buflen]), len (buflen)
  {
    if (buf)
      memcpy (data, buf, len);
  }
};

enum ns_stat {
  NS_OK = 0,
  NS_ERR = 1,
  NS_NOENT = 2,
};

typedef callback<void, ns_stat, bigint>::ref nscb_insert_t;
typedef callback<void, ptr<ns_block> >::ref nscb_retrieve_t;

class ns
{
public:
  static void init();
  static void stat();

  static void retrieve_block   (bigint id, nscb_retrieve_t cb);
  static void retrieve_record  (bigint id, cb_recordp cb);
  static void retrieve_loghead (bigint id, cb_recordp cb);

  static void insert_block (bigint, const char *, size_t, nscb_insert_t cb);
  static void insert_block (const char *, size_t, nscb_insert_t cb);
  
  static void insert_record (bigint, const char *, size_t, nscb_insert_t cb,
			     recordp r=0);
  static void insert_record (const char *, size_t, nscb_insert_t cb,
                             recordp r=0);

  static void insert_loghead (bigint, sfs_pubkey2, sfs_sig2,
                              const char *, size_t, nscb_insert_t,
			      recordp r = 0);
  static void insert_loghead (sfs_pubkey2, sfs_sig2, const char *, size_t,
                              nscb_insert_t, recordp r = 0);

  static bool has_block  (bigint);
  static bool has_record (bigint);
};

#endif

