/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef PACK_H
#define PACK_H

#include "str.h"
#include "bigint.h"

class xin {
private:
  str _s;
  unsigned int _i;

public:
  xin(str s) : _s(s), _i(0) { };
  bool available (unsigned int x) { return (_s.len() - _i >= x); };
  bool rint (int &);
  bool rquad (unsigned long long &);
  bool rbigint (bigint &);
  bool rstr (str &);
};

inline bool
xin::rint (int &xx)
{
  if (available(4) && sizeof(xx) == 4) {
    bcopy (_s.cstr() + _i, &xx, 4);
    xx = ntohl(xx);
    _i += 4;
    return true;
  }
  else
    return false;
}

inline bool
xin::rquad (unsigned long long &xx)
{
  int a, b;
  if (rint(a) && rint(b)) {
    xx = a;
    xx <<= 32;
    xx |= (b & (unsigned long long) 0xffffffff);
    return true;
  }
  else
    return false;
}

inline bool
xin::rbigint (bigint &id)
{
  if (available (20)) {
    mpz_set_rawmag_be (&id, _s.cstr() + _i, 20);
    _i += 20;
    return true;
  }
  else
    return false;
}

inline bool
xin::rstr (str &s)
{
  int len;
  if (rint(len) && available(len)) {
    s = str(_s.cstr() + _i, len);
    _i += len;
    while ((_i % 4) != 0)
      _i++;
    return true;
  }
  else
    return false;
}

inline void
sbigint (strbuf &sb, bigint id)
{
  char b[20];
  mpz_get_rawmag_be(b, sizeof(b), &id);
  sb.tosuio()->copy(b, 20);
}

inline void
sint (strbuf &sb, int x)
{
  int xx = htonl(x);
  sb.tosuio()->copy(&xx, 4);
}

inline void
squad (strbuf &sb, long long x)
{
  sint(sb, x >> 32);
  sint(sb, x);
}

inline void
sstr (strbuf &sb, str s)
{
  sint(sb, s.len());
  sb.tosuio()->copy(s.cstr(), s.len());
  int z = (s.len() % 4);
  if (z)
    sb.tosuio()->copy("xxxx", 4 - z);
}

#endif

