/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#include <time.h>
#include "definc.h"
#include "record.h"
#include "pack.h"

log_record::log_record (int type)
{
  _record_type = type;
  set_time ();
}

void
log_record::set_time ()
{
  struct timeval t;
  gettimeofday (&t, 0);
  _ts = t.tv_sec;
}

bigint
log_record::hash () const
{
  str data = serialize ();
  return compute_sha1_hash (data.cstr (), data.len ());
}

void
log_record::serialize_meta (strbuf &sb) const
{
  sbigint (sb, _prev);
  sbigint (sb, _head);
  sint (sb, _seqnum);
  sint (sb, _record_type);
  sint (sb, _ts);
  _version.serialize (sb);
}

end_log_record::end_log_record ()
  : log_record (T_END)
{
  set_prev (0);
}

end_log_record::end_log_record (xin *)
{
}

str
end_log_record::serialize () const
{
  strbuf sb;
  serialize_meta (sb);
  return sb;
}

head_log_record::head_log_record (bigint root)
  : log_record (T_HEAD), _snapshot (0)
{
  _root = root;
}

head_log_record::head_log_record ()
  : log_record(), _snapshot (0)
{
}

head_log_record::head_log_record (xin *in)
{
  in->rbigint (_root);
  in->rbigint (_snapshot);
}

str
head_log_record::serialize () const
{
  strbuf sb;
  serialize_meta (sb);
  sbigint (sb, _root);
  sbigint (sb, _snapshot);
  return sb;
}

