/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef VIEW_H
#define VIEW_H

#include "definc.h"
#include "pack.h"
#include "ns.h"

class view_constructor {
private:
  friend class view;
  bigint _root;
  vec<bigint> _head;
  vec<bigint> _latest;
  void sort ();
  void parse (xin *in);

public:
  view_constructor () {}
  view_constructor (const view &);

  void root (bigint n) { _root = n; }
  void add_log (bigint h, bigint b);
  str serialize ();
  bigint store (callback<void, ns_stat, bigint>::ref cb);
};

class view {
private:
  bigint _root;
  vec<bigint> _head;
  vec<bigint> _latest;

  struct view_state {
    vec<bigint> latest;
    vec<unsigned> seqnum;
    unsigned nrecords;
    log_status error;
    view_state () { nrecords = 0; error = LOG_OK; }
  };

  void got_view (ptr<view_constructor> vc, 
                 callback<void, log_status>::ref cb, ptr<ns_block> blk);
  void load_record (bool ishead, ptr<view_state> v, unsigned i,
                    callback<void, vervec, log_status>::ref cb, recordp e);
  void load_done (ptr<view_state> v,
                  callback<void, vervec, log_status>::ref cb);

public:
  view (callback<void, log_status>::ref cb, bigint addr);
  view (ptr<view_constructor> c);
  void take (ptr<view_constructor> c);

  unsigned nlogs () const { return _head.size(); }
  bigint root () const { return _root; }
  bigint head (unsigned i) const { return _head[i]; }
  bigint latest (unsigned i) const { return _latest[i]; }

  void version (callback<void, vervec, log_status>::ref cb);
};

#endif

