/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#include "pastwatch.h"
#include "cfsrw_wrapper.h"

typedef vec<str> strv;
typedef ptr<strv> strvp;

static void
splitpath (vec<str> &out, str in)
{
  const char *p = in.cstr ();
  const char *e = p + in.len ();
  const char *n;

  for (;;) {
    while (*p == '/')
      p++;
    for (n = p; n < e && *n != '/'; n++)
      ;
    if (n == p)
      return;
    out.push_back (str (p, n - p));
    p = n;
  }
}

static void next_lookup (bigint, strvp, cfsrw_cb);

static void
lookup_reply (strvp names, cfsrw_cb cb, cfscallp c, cfsrw_status status)
{
  if (status || names->size () == 0) {
    cb (c, status);
    return;
  }
  cfsrw_lookup_res *res = static_cast<cfsrw_lookup_res *> (c->res ());
  next_lookup (res->object, names, cb);
}

static void
next_lookup (bigint fh, strvp names, cfsrw_cb cb)
{
  str p = names->pop_front ();
  cfsrw_lookup_one (fh, p, wrap (lookup_reply, names, cb));
}

void
cfsrw_lookup (bigint fh, str path, cfsrw_cb cb)
{
  strvp names = New refcounted<strv>;
  splitpath (*names, path);
  assert (names->size ());
  next_lookup (fh, names, cb);
}

