/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef PASTWATCH_CFSRW_H
#define PASTWATCH_CFSRW_H

#include "cfsrw_prot.h"

extern void cfsrw_call (ptr<cfscall> c);

inline void
cfsrw_getattr (bigint object, cfsrw_cb cb)
{
  cfscallp c = New refcounted<cfscall> (CFSRWPROC_GETATTR, cb);
  cfsrw_getattr_arg *arg = static_cast<cfsrw_getattr_arg *> (c->arg ());
  arg->object = object;
  cfsrw_call (c);
}

inline void
cfsrw_readdir (bigint dir, uint64 cookie, cfsrw_cb cb)
{
  cfscallp c = New refcounted<cfscall> (CFSRWPROC_READDIR, cb);
  cfsrw_readdir_arg *arg = static_cast<cfsrw_readdir_arg *> (c->arg ());
  arg->dir = dir;
  arg->cookie = cookie;
  arg->count = 1024;
  cfsrw_call (c);
}

inline void
cfsrw_read (bigint fh, uint64 offset, uint32 count, cfsrw_cb cb)
{
  cfscallp c = New refcounted<cfscall> (CFSRWPROC_READ, cb);
  cfsrw_read_arg *arg = static_cast<cfsrw_read_arg *> (c->arg ());
  arg->file = fh;
  arg->offset = offset;
  arg->count = count;
  cfsrw_call (c);
}

inline void
cfsrw_readlink (bigint fh, cfsrw_cb cb)
{
  cfscallp c = New refcounted<cfscall> (CFSRWPROC_READLINK, cb);
  cfsrw_readlink_arg *arg = static_cast<cfsrw_readlink_arg *> (c->arg ());
  arg->file = fh;
  cfsrw_call (c);
}

inline void
cfsrw_lookup_one (bigint fh, str p, cfsrw_cb cb)
{
  cfscallp c = New refcounted<cfscall> (CFSRWPROC_LOOKUP, cb);
  cfsrw_lookup_arg *arg = static_cast<cfsrw_lookup_arg *> (c->arg ());
  arg->dir = fh;
  arg->name = p;
  cfsrw_call (c);
}

void cfsrw_lookup (bigint fh, str path, cfsrw_cb cb);

#endif

