/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef PASTWATCH_H
#define PASTWATCH_H

#include "definc.h"
#include "cfsrw_prot.h"

#define PASTWATCH_REF "Pastwatch"

struct pde_t {  // pastwatch directory entry
  str name;
  str path;
  str cfspath;
  cfsrw_ftype type;
  bigint fh;
  bigint dirfh;
};

struct mtime_t {
  time_t sec;
  long nsec;
  mtime_t () { sec = 0; nsec = 0; }
};

extern void cfsrw_call (ptr<cfscall> c);
extern void cfsrw_call_at (ptr<cfscall> c, vervec start);
extern void cfsrw_call_latest (ptr<cfscall> c);

typedef callback<void, bigint, int>::ref fhcb_t;
typedef callback<void, const pde_t &, fhcb_t >::ref recursecb_t;
extern void recurse (str dirpath, recursecb_t iter_cb, cbi cb, bigint dirfh);

extern const int cmd_checkin;
extern const int cmd_checkout;
extern const int cmd_update;
extern const int cmd_commit;
extern const int cmd_add;
extern const int cmd_remove;
extern const int cmd_rename;
extern const int cmd_clear;

extern bool arg_updatedir;
extern bool arg_checkonly;
extern bool arg_debugmode;

inline str
pastwatch_tmpfile ()
{
  // create tmp file
  char raw[20];
  rnd.getbytes(raw, 20);
  str name = armor64A (raw, sizeof (raw));
  strbuf b;
  b << "/tmp/pastwatch." << name;
  str s = b;
  return s;
}

inline void
pastwatch_status (str s, str path)
{
  strbuf b;
  if (s != "")
    b << s << " ";
  b << path << "\n";
  b.tosuio ()->output (1);
}

inline void
pastwatch_status (str s, str path, str msg)
{
  strbuf b;
  if (s != "")
    b << s << " ";
  b << path << ": " << msg << "\n";
  b.tosuio ()->output (1);
}

inline str
make_cfspath (str cfsdirpath, str file)
{
  str cfspath;
  if (cfsdirpath != "")
    cfspath = cfsdirpath << "/" << file;
  else
    cfspath = file;
  return cfspath;
}

#endif

