/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef PASTWATCH_PATH_H
#define PASTWATCH_PATH_H

#include "definc.h"

extern str arg_viewstr;

inline str
sanitize_path (str path)
{
  unsigned end = path.len ()-1;
  // remove trailing /
  while (path[end] == '/' && end > 0)
    end--;
  if (end == 0 && path[end] == '/') // root
    return "/";

  char s [end+1];
  unsigned i,j;
  for (i=0, j=0; i <= end;) {
    s [j] = path [i];
    j++;
    if (path [i] == '/') {
      while (i <= end && path [i] == '/')
        i++;
    }
    else
      i++;
  }
  return str (s, j);
}

inline str
get_parent_dir (str path)
{
  if (path == "/")
    return "";

  int i = path.len ()-1;
  while (i >= 0) {
    if (path [i] == '/')
      break;
    i--;
  }
  str dir;
  if (i < 0)
    dir = ".";
  else
    dir = substr (path, 0, i);
  return dir;
}

inline str
get_dir (str path)
{
  struct stat sb;
  if (lstat (path, &sb) == 0 && (sb.st_mode & S_IFDIR))
    return path;
  return get_parent_dir (path);
}

inline str
get_entries_file (str dir)
{
  str s = dir << "/" << PASTWATCH_REF << "/" << arg_viewstr << ".entries";
  return s;
}

inline str
get_db_file (str dir)
{
  str s = dir << "/" << PASTWATCH_REF << "/" << arg_viewstr << ".db";
  return s;
}

extern int get_cfspath (str path, str &cfspath);

#endif

