/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#include <sys/types.h>
#include <dirent.h>
#include "definc.h"
#include "pastwatch.h"

class recurse_object : public virtual refcount
{
  int work;
  int error;

  void
  recurse_reply (str path, recursecb_t iter_cb, cbi cb, bool dir,
               bigint fh, int err)
  {
    work --;
    if (err) {
      error = -1;
      if (work == 0)
        cb (error);
      return;
    }
    if (dir)
      do_recurse (path, iter_cb, cb, fh);
    if (work == 0)
      cb (error);
  }
  
  void
  do_recurse (str dirpath, recursecb_t iter_cb, cbi cb, bigint dirfh)
  {
    work ++;
    
    DIR *dir;
    dir = opendir (dirpath);
  
    if (dir == 0) {
      work --;
      error = -1;
      perror ("opendir");
      if (work == 0)
        cb (error);
      return;
    }
  
    struct dirent *e;
    while ((e = readdir (dir)) && !error) {
      if (e->d_type != DT_DIR && e->d_type != DT_REG && e->d_type != DT_LNK) {
        warn << "skipping " << e->d_name << ": file type "
	     << e->d_type << " not supported\n";
        continue;
      }
      if (strcmp (e->d_name, ".") && strcmp (e->d_name, "..") && 
	  strcmp (e->d_name, PASTWATCH_REF)) {
        strbuf b = dirpath << "/" << e->d_name;
	pde_t pde;
	pde.name = str (e->d_name, e->d_namlen);
	pde.path = b;
	if (e->d_type == DT_DIR)
	  pde.type = CFSRW_DIR;
	else if (e->d_type == DT_REG)
	  pde.type = CFSRW_REG;
	else if (e->d_type == DT_LNK)
	  pde.type = CFSRW_LNK;
        pde.dirfh = dirfh;
        work ++;
        iter_cb 
	  (pde, wrap (mkref(this), &recurse_object::recurse_reply, pde.path,
		      iter_cb, cb, e->d_type == DT_DIR));
      }
    }
  
    closedir (dir);
    work --;
    if (work == 0)
      cb (error);
  }

public:
  recurse_object (str dirpath, recursecb_t iter_cb, cbi cb, bigint dirfh)
    : work (0), error (0)
  {
    do_recurse (dirpath, iter_cb, cb, dirfh);
  }

};

void
recurse (str dirpath, recursecb_t iter_cb, cbi cb, bigint dirfh)
{
  ptr<recurse_object> a =
    New refcounted<recurse_object> (dirpath, iter_cb, cb, dirfh);
}

