/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#include "snapshot_spec.h"
#include "pastwatch.h"

static vec <bigint> snapshot_addrs;
static vec <vervec> snapshot_versions;
static off_t infofile_offset = 0;

extern str arg_keystr;
extern str arg_viewstr;

static int
read_one_entry (int fd, off_t &offset, bigint &addr, vervec &v)
{
  int len = 0;
  if (lseek (fd, offset-sizeof (int), SEEK_SET) < 0 ||
      read (fd, &len, sizeof (int)) != sizeof (int) ||
      len < 0 ||
      sizeof (int)+len > offset)
    return -1;

  char buf[len];
  if (lseek (fd, offset-sizeof (int)-len, SEEK_SET) < 0 ||
      read (fd, buf, len) <= 0)
    return -1;

  str s (buf, len);
  xin x (s);
  x.rbigint (addr);
  if (vervec::parse (&x, v))
    return -1;
  offset -= (sizeof (int)+len);
  return 0;
}

void
snapshot_find (ptr<cfscall> c, vervec start, cb_snapshot cb)
{
  bool found = false;

  for (unsigned i=0; i<snapshot_addrs.size (); i++) {
    if (start.gt (snapshot_versions [i])) {
      snapshot_operation::load_snapshot (snapshot_addrs [i], cb);
      return;
    }
  }
  if (snapshot_addrs.size ()) {
    cb (0, (ptr<snapshot_obj>) 0);
    return;
  }

  char *home = getenv ("HOME");
  str fn = str (home, strlen (home)) << "/.pastwatch/" << 
           arg_keystr << "/" << arg_viewstr << "/" << "snapshots";
  int fd = open (fn, O_RDONLY);
  if (fd < 0) {
    cb (0, (ptr<snapshot_obj>) 0);
    return;
  }

  if (snapshot_addrs.size ())
    infofile_offset = lseek (fd, infofile_offset, SEEK_SET);
  else
    infofile_offset = lseek (fd, 0, SEEK_END);
  if (infofile_offset < 0) {
    cb (0, (ptr<snapshot_obj>) 0);
    return;
  }

  while (infofile_offset >= sizeof (int)) {
    bigint addr;
    vervec v;

    if (read_one_entry (fd, infofile_offset, addr, v))
      break;
    
    if (arg_debugmode) {
      warn << "addr: " << addr << "\n";
      warn << "start: " << start.seqnum (0) 
           << ", v: " << v.seqnum (0) << "\n";
    }

    snapshot_addrs.push_back (addr);
    snapshot_versions.push_back (v);
    
    if (!found && start.gt (v)) {
      found = true;
      snapshot_operation::load_snapshot (addr, cb);
    }
  }

  close (fd);
  if (!found)
    cb (0, (ptr<snapshot_obj>) 0);
  return;
}

void
snapshot_add (bigint addr, vervec version)
{
  char *home = getenv ("HOME");
  str fn = str (home, strlen (home)) << "/.pastwatch/" << 
           arg_keystr << "/" << arg_viewstr << "/" << "snapshots";
  int fd = open (fn, O_CREAT | O_RDWR, 0644);
  if (fd) {
    off_t off = lseek (fd, 0, SEEK_END);
    if (off < 0)
      return;

    if (off > 0) {
      if (off < sizeof (int))
        return;
      bigint a;
      vervec v;
      if (read_one_entry (fd, off, a, v) ||
	  v.gt (version) ||
	  lseek (fd, 0, SEEK_END) < 0)
        return;
    }

    strbuf p;
    sbigint (p, addr);
    version.serialize (p);
    str s = p;
    int n = s.len ();
    write (fd, s.cstr (), s.len ());
    write (fd, &n, sizeof (int));
    close (fd);
  }
}

