/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef PASTWATCH_VDB_H
#define PASTWATCH_VDB_H

#include "dbfe.h"
#include "cfsrw_prot.h"

struct pfe_t { // pastwatch file entry
  bigint fh;
  vervec version;
  mtime_t mtime;

  pfe_t (bigint h, vervec v) : fh (h), version (v)
  {
    mtime.sec = 0;
    mtime.nsec = 0;
  }
  pfe_t (bigint h, vervec v, mtime_t m) : fh (h), version (v), mtime (m) {}

  ptr<dbrec> to_dbrec () const {
    strbuf b;
    sbigint (b, fh);
    version.serialize (b);
    str s = b;
  
    int len = sizeof (mtime_t)+s.len ();
    char buf[len];
    memmove (buf, &(mtime), sizeof (mtime_t));
    memmove (buf+sizeof (mtime_t), s.cstr (), s.len ());
    return New refcounted<dbrec> (&buf[0], len);
  }
};

struct pfe_raw {
  ptr<dbrec> v;
  pfe_raw (ptr<dbrec> p) : v(p) {}

  void mtime (mtime_t &m) { 
    memmove (v->value, &m, sizeof (mtime_t));
  }
  mtime_t mtime () const {
    mtime_t m;
    memmove (&m, v->value, sizeof (mtime_t));
    return m;
  }
  bigint fh () const {
    str s (v->value+sizeof (mtime_t), v->len-sizeof (mtime_t));
    bigint n;
    xin x (s);
    x.rbigint (n);
    return n;
  }
  vervec version () const {
    str s (v->value+sizeof (mtime_t), v->len-sizeof (mtime_t));
    bigint n;
    xin x (s);
    x.rbigint (n);
    vervec v;
    vervec::parse (&x, v);
    return v;
  }
};

extern int vdb_init ();
extern int vdb_process ();

extern bool vdb_exists (str path);
extern int  vdb_remove (str path);
extern int  vdb_insert (str path, pfe_t);
extern ptr<pfe_raw> vdb_lookup (str path);

extern void vdb_schedule_mkdir (str path, str cfspath);
extern void vdb_schedule_add (str path, pfe_t);
extern void vdb_schedule_remove (str path);

extern int remove_dir (str path);

#endif

