/*
 *  Copyright (C) 2002-2003  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#include "pastwatch.h"

#define CONCURRENT_WRITES 16

class write_obj
{
  str path;
  bigint fh;
  callback<void, cfsrw_status>::ref cb;
 
  int fd;
  uint64 woff;
  unsigned wctr;
  unsigned werr;

  void
  write_reply (cfscallp c, cfsrw_status status)
  {
    wctr--;
    if (status) {
      werr = 1;
      done ();
    }
    else
      do_write ();
  }
  
  void do_write ()
  {
    int r = 0;
    char buf [4096];
    while (wctr < CONCURRENT_WRITES && (r = read (fd, buf, 4096)) > 0) {
      cfscallp c = New refcounted<cfscall>
	(CFSRWPROC_WRITE, wrap (this, &write_obj::write_reply));
      cfsrw_write_arg *arg = static_cast<cfsrw_write_arg *> (c->arg ());
      arg->file = fh;
      arg->offset = woff;
      arg->count = r;
      str data (buf, r);
      arg->data = data;
      woff += r;
      wctr++;
      cfsrw_call (c);
    }
    if (r < 0) {
      perror ("read");
      werr = 1;
    }
    done ();
  }

  void done ()
  {
    if (wctr == 0) {
      if (werr)
        cb (CFSRWERR_IO);
      else
        cb (CFSRW_OK);
      close (fd);
      delete this;
    }
  }

public:
  write_obj (str path, bigint fh, callback<void, cfsrw_status>::ref cb)
    : path(path), fh(fh), cb(cb)
  {
    woff = 0;
    wctr = 0;
    werr = 0;
  
    fd = open (path, O_RDONLY);
    if (fd < 0) {
      cb (CFSRWERR_IO);
      return;
    }
    do_write ();
  }
};


void
write_file (str path, bigint fh, callback<void, cfsrw_status>::ref cb)
{
  vNew write_obj (path, fh, cb);
}

